const state = {
    cartItems: [],
    cartTotal: 0,
    itemPriced: null
}

const getters = {
    /**
     * 
     * @param {*} state 
     */
    cartItems(state) {
        return state.cartItems
    },
    /**
     * 
     * @param {*} state 
     */
    cartTotal(state) {
        return state.cartTotal
    },
    /**
     * 
     * @param {*} state 
     */
    itemPriced(state) {
        return state.itemPriced
    }
}

const mutations = {
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setCartItems(state, payload) {
        state.cartItems= [...payload]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    pushToCart(state, payload) {
        let productFound = state.cartItems.find(item => item.id === payload.id)

        if(productFound) {
            if (payload.quantity > 1) {
                if (productFound.quantity === payload.quantity) {
                    productFound.quantity = productFound.quantity + 1    
                } else {
                    productFound.quantity = payload.quantity
                }
            } else {
                productFound.quantity = productFound.quantity + 1
            }
            return
        }

        state.cartItems = [payload, ...state.cartItems]
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    pullFromCart(state, payload) {
        let foundIndex = state.cartItems.findIndex(item => item.id === payload.item.id)

        if(state.cartItems[foundIndex].quantity === 1) {
            payload.mode = 'full'
        }

        if (payload.mode === 'partial') {
            if (payload.item.quantity !== -1) {
                state.cartItems[foundIndex].quantity = payload.item.quantity
            } else {
                state.cartItems[foundIndex].quantity = state.cartItems[foundIndex].quantity - 1
            }
        } else {
            state.cartItems.splice(foundIndex, 1)
        }
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setCartTotal(state, payload) {
        state.cartTotal = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    setItemPriced(state, payload) {
        state.itemPriced = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    applyPrice(state, payload) {
        let foundProduct = state.cartItems.find(item => item.id === state.itemPriced.id)
        foundProduct.price = payload
    },
    /**
     * 
     * @param {*} state 
     * @param {*} payload 
     */
    undoPrice(state, payload) {
        let foundProduct = state.cartItems.find(item => item.id === payload.id)
        foundProduct.price = foundProduct.standardPrice
    }
}

const actions = {
    /**
     * Add item to cart or increment item quantity if exist
     * @param {*} param0 
     * @param {*} payload 
     */
    addToCart({ commit }, payload) {
        commit('pushToCart', payload)
    },
    /**
     * Remove item from cart
     * @param {*} param0 
     * @param {*} payload 
     */
    removeFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'full'
        })
    },
    /**
     * Decrement item quantity from cart
     * @param {*} param0 
     * @param {*} payload 
     */
    decreaseFromCart({ commit }, payload) {
        commit('pullFromCart', {
            item: payload,
            mode: 'partial'
        })
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    changePrice({ commit }, payload) {
        if (typeof payload === 'number') {
           commit('applyPrice', payload)
        }

        if (typeof payload === 'object') {
            commit('setItemPriced', payload)
            return
        }

        commit('setItemPriced', null)

    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    undoPrice({ commit }, payload) {
        commit('undoPrice', payload)
    },
    /**
     * 
     * @param {*} param0 
     * @param {*} payload 
     */
    updateCartTotal({ commit }, payload) {
        commit('setCartTotal', payload)
    },
    /**
     * 
     * @param {*} param0 
     */
    resetCart({ commit }) {
        commit('setCartItems', [])
        commit('setCartTotal', 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}